
public class PL3mmJomp {


	public static int threadNumber = 4;
	/* Array initialization. */
	public static void init_array(int ni, int nj, int nk, int nl, int nm,
			double[][] A, double[][] B, double[][] C, double[][] D) {
		int i;
		int j;

		for (i = 0; i < ni; i++) {
			for (j = 0; j < nk; j++) {
				A[i][j] = ((double) i * j) / ni;
			}
		}
		for (i = 0; i < nk; i++) {
			for (j = 0; j < nj; j++) {
				B[i][j] = ((double) i * (j + 1)) / nj;
			}
		}
		for (i = 0; i < nj; i++) {
			for (j = 0; j < nm; j++) {
				C[i][j] = ((double) i * (j + 3)) / nl;
			}
		}
		for (i = 0; i < nm; i++) {
			for (j = 0; j < nl; j++) {
				D[i][j] = ((double) i * (j + 2)) / nk;
			}
		}
	}

	/*
	 * DCE code. Must scan the entire live-out data. Can be used also to check
	 * the correctness of the output.
	 */
	public static void print_array(int ni, int nl, double[][] G) {
		int i;
		int j;

		for (i = 0; i < ni; i++) {
			for (j = 0; j < nl; j++) {
				System.out.printf("%.2f ", G[i][j]);
				if ((i * ni + j) % 20 == 0) {
					System.out.println();
				}
			}
		}
		System.out.println();
	}

	/*
	 * Main computational kernel. The whole function will be timed, including
	 * the call and return.
	 */
	public static void kernel_3mm(int ni, int nj, int nk, int nl, int nm,
			double[][] E, double[][] A, double[][] B, double[][] F,
			double[][] C, double[][] D, double[][] G) {

// OMP PARALLEL BLOCK BEGINS
{
  __omp_Class0 __omp_Object0 = new __omp_Class0();
  // shared variables
  __omp_Object0.ni = ni;
  __omp_Object0.nj = nj;
  __omp_Object0.nk = nk;
  __omp_Object0.nl = nl;
  __omp_Object0.nm = nm;
  __omp_Object0.A = A;
  __omp_Object0.B = B;
  __omp_Object0.C = C;
  __omp_Object0.D = D;
  __omp_Object0.E = E;
  __omp_Object0.F = F;
  __omp_Object0.G = G;
  // firstprivate variables
  try {
    jomp.runtime.OMP.doParallel(__omp_Object0);
  } catch(Throwable __omp_exception) {
    System.err.println("OMP Warning: Illegal thread exception ignored!");
    System.err.println(__omp_exception);
  }
  // reduction variables
  // shared variables
  ni = __omp_Object0.ni;
  nj = __omp_Object0.nj;
  nk = __omp_Object0.nk;
  nl = __omp_Object0.nl;
  nm = __omp_Object0.nm;
  A = __omp_Object0.A;
  B = __omp_Object0.B;
  C = __omp_Object0.C;
  D = __omp_Object0.D;
  E = __omp_Object0.E;
  F = __omp_Object0.F;
  G = __omp_Object0.G;
}
// OMP PARALLEL BLOCK ENDS

		// #pragma endscop

	}

	public static void main(String[] args) {
		/* Retrieve problem size. */
		String dataSet;
		if (args.length == 2) {
			dataSet = args[0].toUpperCase();
			threadNumber = Integer.parseInt(args[1]);
		} else if (args.length == 1) {
			dataSet = args[0].toUpperCase();
		} else {
			dataSet = "STANDARD";
		}
		int NI, NJ, NK, NL, NM;
		switch (dataSet) {
		case "MINI":
			NI = 32;
			NJ = 32;
			NK = 32;
			NL = 32;
			NM = 32;
			break;
		case "SMALL":
			NI = 128;
			NJ = 128;
			NK = 128;
			NL = 128;
			NM = 128;
			break;
		case "STANDARD":
		default:
			NI = 1024;
			NJ = 1024;
			NK = 1024;
			NL = 1024;
			NM = 1024;
			break;
		case "LARGE":
			NI = 2000;
			NJ = 2000;
			NK = 2000;
			NL = 2000;
			NM = 2000;
			break;
		case "EXTRALARGE":
			NI = 4000;
			NJ = 4000;
			NK = 4000;
			NL = 4000;
			NM = 4000;
			break;
		}
		int ni = NI;
		int nj = NJ;
		int nk = NK;
		int nl = NL;
		int nm = NM;

		/* Variable declaration/allocation. */
		double[][] E = new double[ni][nj];
		double[][] A = new double[ni][nk];
		double[][] B = new double[nk][nj];
		double[][] F = new double[nj][nl];
		double[][] C = new double[nj][nm];
		double[][] D = new double[nm][nl];
		double[][] G = new double[ni][nl];

		/* Initialize array(s). */
		PL3mmJomp.init_array(ni, nj, nk, nl, nm, A, B, C, D);

		/* Start timer. */
		long pre = System.currentTimeMillis();

		/* Run kernel. */
		PL3mmJomp.kernel_3mm(ni, nj, nk, nl, nm, E, A, B, F, C, D, G);

		/* Stop and print timer. */
		long pos = System.currentTimeMillis();
		System.out.print(pos - pre);

		/*
		 * Prevent dead-code elimination. All live-out data must be printed by
		 * the function call in argument.
		 */
		// PL3mmJomp.print_array(ni, nl, G);

	}

// OMP PARALLEL REGION INNER CLASS DEFINITION BEGINS
private static class __omp_Class0 extends jomp.runtime.BusyTask {
  // shared variables
  int ni;
  int nj;
  int nk;
  int nl;
  int nm;
  double [ ] [ ] A;
  double [ ] [ ] B;
  double [ ] [ ] C;
  double [ ] [ ] D;
  double [ ] [ ] E;
  double [ ] [ ] F;
  double [ ] [ ] G;
  // firstprivate variables
  // variables to hold results of reduction

  public void go(int __omp_me) throws Throwable {
  // firstprivate variables + init
  // private variables
  // reduction variables, init to default
    // OMP USER CODE BEGINS
 
		{
                 { // OMP FOR BLOCK BEGINS
                 // copy of firstprivate variables, initialized
                 // copy of lastprivate variables
                 // variables to hold result of reduction
                 boolean amLast=false;
                 {
                   // firstprivate variables + init
                   // [last]private variables
                   // reduction variables + init to default
                   // -------------------------------------
                   jomp.runtime.LoopData __omp_WholeData2 = new jomp.runtime.LoopData();
                   jomp.runtime.LoopData __omp_ChunkData1 = new jomp.runtime.LoopData();
                   __omp_WholeData2.start = (long)( 0);
                   __omp_WholeData2.stop = (long)( ni);
                   __omp_WholeData2.step = (long)(1);
                   jomp.runtime.OMP.setChunkStatic(__omp_WholeData2);
                   while(!__omp_ChunkData1.isLast && jomp.runtime.OMP.getLoopStatic(__omp_me, __omp_WholeData2, __omp_ChunkData1)) {
                   for(;;) {
                     if(__omp_WholeData2.step > 0) {
                        if(__omp_ChunkData1.stop > __omp_WholeData2.stop) __omp_ChunkData1.stop = __omp_WholeData2.stop;
                        if(__omp_ChunkData1.start >= __omp_WholeData2.stop) break;
                     } else {
                        if(__omp_ChunkData1.stop < __omp_WholeData2.stop) __omp_ChunkData1.stop = __omp_WholeData2.stop;
                        if(__omp_ChunkData1.start > __omp_WholeData2.stop) break;
                     }
                     for(int i = (int)__omp_ChunkData1.start; i < __omp_ChunkData1.stop; i += __omp_ChunkData1.step) {
                       // OMP USER CODE BEGINS
 {
			for (int j = 0; j < nj; j++) {
				E[i][j] = 0;
				for (int k = 0; k < nk; ++k) {
					E[i][j] += A[i][k] * B[k][j];
				}
			}
		}
                       // OMP USER CODE ENDS
                       if (i == (__omp_WholeData2.stop-1)) amLast = true;
                     } // of for 
                     if(__omp_ChunkData1.startStep == 0)
                       break;
                     __omp_ChunkData1.start += __omp_ChunkData1.startStep;
                     __omp_ChunkData1.stop += __omp_ChunkData1.startStep;
                   } // of for(;;)
                   } // of while
                   // call reducer
                   jomp.runtime.OMP.doBarrier(__omp_me);
                   // copy lastprivate variables out
                   if (amLast) {
                   }
                 }
                 // set global from lastprivate variables
                 if (amLast) {
                 }
                 // set global from reduction variables
                 if (jomp.runtime.OMP.getThreadNum(__omp_me) == 0) {
                 }
                 } // OMP FOR BLOCK ENDS

                 { // OMP FOR BLOCK BEGINS
                 // copy of firstprivate variables, initialized
                 // copy of lastprivate variables
                 // variables to hold result of reduction
                 boolean amLast=false;
                 {
                   // firstprivate variables + init
                   // [last]private variables
                   // reduction variables + init to default
                   // -------------------------------------
                   jomp.runtime.LoopData __omp_WholeData5 = new jomp.runtime.LoopData();
                   jomp.runtime.LoopData __omp_ChunkData4 = new jomp.runtime.LoopData();
                   __omp_WholeData5.start = (long)( 0);
                   __omp_WholeData5.stop = (long)( nj);
                   __omp_WholeData5.step = (long)(1);
                   jomp.runtime.OMP.setChunkStatic(__omp_WholeData5);
                   while(!__omp_ChunkData4.isLast && jomp.runtime.OMP.getLoopStatic(__omp_me, __omp_WholeData5, __omp_ChunkData4)) {
                   for(;;) {
                     if(__omp_WholeData5.step > 0) {
                        if(__omp_ChunkData4.stop > __omp_WholeData5.stop) __omp_ChunkData4.stop = __omp_WholeData5.stop;
                        if(__omp_ChunkData4.start >= __omp_WholeData5.stop) break;
                     } else {
                        if(__omp_ChunkData4.stop < __omp_WholeData5.stop) __omp_ChunkData4.stop = __omp_WholeData5.stop;
                        if(__omp_ChunkData4.start > __omp_WholeData5.stop) break;
                     }
                     for(int i = (int)__omp_ChunkData4.start; i < __omp_ChunkData4.stop; i += __omp_ChunkData4.step) {
                       // OMP USER CODE BEGINS
 {
			for (int j = 0; j < nl; j++) {
				F[i][j] = 0;
				for (int k = 0; k < nm; ++k) {
					F[i][j] += C[i][k] * D[k][j];
				}
			}
		}
                       // OMP USER CODE ENDS
                       if (i == (__omp_WholeData5.stop-1)) amLast = true;
                     } // of for 
                     if(__omp_ChunkData4.startStep == 0)
                       break;
                     __omp_ChunkData4.start += __omp_ChunkData4.startStep;
                     __omp_ChunkData4.stop += __omp_ChunkData4.startStep;
                   } // of for(;;)
                   } // of while
                   // call reducer
                   jomp.runtime.OMP.doBarrier(__omp_me);
                   // copy lastprivate variables out
                   if (amLast) {
                   }
                 }
                 // set global from lastprivate variables
                 if (amLast) {
                 }
                 // set global from reduction variables
                 if (jomp.runtime.OMP.getThreadNum(__omp_me) == 0) {
                 }
                 } // OMP FOR BLOCK ENDS

                 { // OMP FOR BLOCK BEGINS
                 // copy of firstprivate variables, initialized
                 // copy of lastprivate variables
                 // variables to hold result of reduction
                 boolean amLast=false;
                 {
                   // firstprivate variables + init
                   // [last]private variables
                   // reduction variables + init to default
                   // -------------------------------------
                   jomp.runtime.LoopData __omp_WholeData8 = new jomp.runtime.LoopData();
                   jomp.runtime.LoopData __omp_ChunkData7 = new jomp.runtime.LoopData();
                   __omp_WholeData8.start = (long)( 0);
                   __omp_WholeData8.stop = (long)( ni);
                   __omp_WholeData8.step = (long)(1);
                   jomp.runtime.OMP.setChunkStatic(__omp_WholeData8);
                   while(!__omp_ChunkData7.isLast && jomp.runtime.OMP.getLoopStatic(__omp_me, __omp_WholeData8, __omp_ChunkData7)) {
                   for(;;) {
                     if(__omp_WholeData8.step > 0) {
                        if(__omp_ChunkData7.stop > __omp_WholeData8.stop) __omp_ChunkData7.stop = __omp_WholeData8.stop;
                        if(__omp_ChunkData7.start >= __omp_WholeData8.stop) break;
                     } else {
                        if(__omp_ChunkData7.stop < __omp_WholeData8.stop) __omp_ChunkData7.stop = __omp_WholeData8.stop;
                        if(__omp_ChunkData7.start > __omp_WholeData8.stop) break;
                     }
                     for(int i = (int)__omp_ChunkData7.start; i < __omp_ChunkData7.stop; i += __omp_ChunkData7.step) {
                       // OMP USER CODE BEGINS
 {
			for (int j = 0; j < nl; j++) {
				G[i][j] = 0;
				for (int k = 0; k < nj; ++k) {
					G[i][j] += E[i][k] * F[k][j];
				}
			}
		}
                       // OMP USER CODE ENDS
                       if (i == (__omp_WholeData8.stop-1)) amLast = true;
                     } // of for 
                     if(__omp_ChunkData7.startStep == 0)
                       break;
                     __omp_ChunkData7.start += __omp_ChunkData7.startStep;
                     __omp_ChunkData7.stop += __omp_ChunkData7.startStep;
                   } // of for(;;)
                   } // of while
                   // call reducer
                   jomp.runtime.OMP.doBarrier(__omp_me);
                   // copy lastprivate variables out
                   if (amLast) {
                   }
                 }
                 // set global from lastprivate variables
                 if (amLast) {
                 }
                 // set global from reduction variables
                 if (jomp.runtime.OMP.getThreadNum(__omp_me) == 0) {
                 }
                 } // OMP FOR BLOCK ENDS

		}
    // OMP USER CODE ENDS
  // call reducer
  // output to _rd_ copy
  if (jomp.runtime.OMP.getThreadNum(__omp_me) == 0) {
  }
  }
}
// OMP PARALLEL REGION INNER CLASS DEFINITION ENDS

}

