
public class PLdurbinJomp {


	public static int threadNumber = 4;
	/* Array initialization. */
	public static void init_array(int n, double[][] y, double[][] sum,
			double[] alpha, double[] beta, double[] r) {
		int i;
		int j;

		for (i = 0; i < n; i++) {
			alpha[i] = i;
			beta[i] = (i + 1) / n / 2.0;
			r[i] = (i + 1) / n / 4.0;
			for (j = 0; j < n; j++) {
				y[i][j] = ((double) i * j) / n;
				sum[i][j] = ((double) i * j) / n;
			}
		}
	}

	/*
	 * DCE code. Must scan the entire live-out data. Can be used also to check
	 * the correctness of the output.
	 */
	public static void print_array(int n, double[] out)

	{
		int i;

		for (i = 0; i < n; i++) {
			System.out.printf("%.2f ", out[i]);
			if (i % 20 == 0) {
				System.out.println();
			}
		}
	}

	/*
	 * Main computational kernel. The whole function will be timed, including
	 * the call and return.
	 */
	public static void kernel_durbin(int n, double[][] y, double[][] sum,
			double[] alpha, double[] beta, double[] r, double[] out) {

// OMP PARALLEL BLOCK BEGINS
{
  __omp_Class0 __omp_Object0 = new __omp_Class0();
  // shared variables
  __omp_Object0.y = y;
  __omp_Object0.beta = beta;
  __omp_Object0.alpha = alpha;
  __omp_Object0.n = n;
  __omp_Object0.r = r;
  __omp_Object0.sum = sum;
  __omp_Object0.out = out;
  // firstprivate variables
  try {
    jomp.runtime.OMP.doParallel(__omp_Object0);
  } catch(Throwable __omp_exception) {
    System.err.println("OMP Warning: Illegal thread exception ignored!");
    System.err.println(__omp_exception);
  }
  // reduction variables
  // shared variables
  y = __omp_Object0.y;
  beta = __omp_Object0.beta;
  alpha = __omp_Object0.alpha;
  n = __omp_Object0.n;
  r = __omp_Object0.r;
  sum = __omp_Object0.sum;
  out = __omp_Object0.out;
}
// OMP PARALLEL BLOCK ENDS

		// #pragma endscop

	}

	public static void main(String[] args) {
		/* Retrieve problem size. */
		String dataSet;
		if (args.length == 2) {
			dataSet = args[0].toUpperCase();
			threadNumber = Integer.parseInt(args[1]);
		} else if (args.length == 1) {
			dataSet = args[0].toUpperCase();
		} else {
			dataSet = "STANDARD";
		}
		int N;
		switch (dataSet) {
		case "MINI":
			N = 32;
			break;
		case "SMALL":
			N = 500;
			break;
		case "STANDARD":
		default:
			N = 4000;
			break;
		case "LARGE":
			N = 8000;
			break;
		case "EXTRALARGE":
			N = 100000;
			break;
		}
		int n = N;

		/* Variable declaration/allocation. */
		double[][] y = new double[n][n];
		double[][] sum = new double[n][n];
		double[] alpha = new double[n];
		double[] beta = new double[n];
		double[] r = new double[n];
		double[] out = new double[n];

		/* Initialize array(s). */
		PLdurbinJomp.init_array(n, y, sum, alpha, beta, r);

		/* Start timer. */
		long pre = System.currentTimeMillis();

		/* Run kernel. */
		PLdurbinJomp.kernel_durbin(n, y, sum, alpha, beta, r, out);

		/* Stop and print timer. */
		long pos = System.currentTimeMillis();
		System.out.print(pos - pre);

		/*
		 * Prevent dead-code elimination. All live-out data must be printed by
		 * the function call in argument.
		 */
		// PLdurbinJomp.print_array(n, out);

	}

// OMP PARALLEL REGION INNER CLASS DEFINITION BEGINS
private static class __omp_Class0 extends jomp.runtime.BusyTask {
  // shared variables
  double [ ] [ ] y;
  double [ ] beta;
  double [ ] alpha;
  int n;
  double [ ] r;
  double [ ] [ ] sum;
  double [ ] out;
  // firstprivate variables
  // variables to hold results of reduction

  public void go(int __omp_me) throws Throwable {
  // firstprivate variables + init
  // private variables
  // reduction variables, init to default
    // OMP USER CODE BEGINS
 
                {
		y[0][0] = r[0];
		beta[0] = 1;
		alpha[0] = r[0];
                 { // OMP FOR BLOCK BEGINS
                 // copy of firstprivate variables, initialized
                 // copy of lastprivate variables
                 // variables to hold result of reduction
                 boolean amLast=false;
                 {
                   // firstprivate variables + init
                   // [last]private variables
                   // reduction variables + init to default
                   // -------------------------------------
                   jomp.runtime.LoopData __omp_WholeData2 = new jomp.runtime.LoopData();
                   jomp.runtime.LoopData __omp_ChunkData1 = new jomp.runtime.LoopData();
                   __omp_WholeData2.start = (long)( 1);
                   __omp_WholeData2.stop = (long)( n);
                   __omp_WholeData2.step = (long)(1);
                   jomp.runtime.OMP.setChunkStatic(__omp_WholeData2);
                   while(!__omp_ChunkData1.isLast && jomp.runtime.OMP.getLoopStatic(__omp_me, __omp_WholeData2, __omp_ChunkData1)) {
                   for(;;) {
                     if(__omp_WholeData2.step > 0) {
                        if(__omp_ChunkData1.stop > __omp_WholeData2.stop) __omp_ChunkData1.stop = __omp_WholeData2.stop;
                        if(__omp_ChunkData1.start >= __omp_WholeData2.stop) break;
                     } else {
                        if(__omp_ChunkData1.stop < __omp_WholeData2.stop) __omp_ChunkData1.stop = __omp_WholeData2.stop;
                        if(__omp_ChunkData1.start > __omp_WholeData2.stop) break;
                     }
                     for(int k = (int)__omp_ChunkData1.start; k < __omp_ChunkData1.stop; k += __omp_ChunkData1.step) {
                       // OMP USER CODE BEGINS
 {
			beta[k] = beta[k - 1] - alpha[k - 1] * alpha[k - 1] * beta[k - 1];
			sum[0][k] = r[k];
			for (int i = 0; i <= k - 1; i++) {
				sum[i + 1][k] = sum[i][k] + r[k - i - 1] * y[i][k - 1];
			}
			alpha[k] = -sum[k][k] * beta[k];
			for (int i = 0; i <= k - 1; i++) {
				y[i][k] = y[i][k - 1] + alpha[k] * y[k - i - 1][k - 1];
			}
			y[k][k] = alpha[k];
		}
                       // OMP USER CODE ENDS
                       if (k == (__omp_WholeData2.stop-1)) amLast = true;
                     } // of for 
                     if(__omp_ChunkData1.startStep == 0)
                       break;
                     __omp_ChunkData1.start += __omp_ChunkData1.startStep;
                     __omp_ChunkData1.stop += __omp_ChunkData1.startStep;
                   } // of for(;;)
                   } // of while
                   // call reducer
                   jomp.runtime.OMP.doBarrier(__omp_me);
                   // copy lastprivate variables out
                   if (amLast) {
                   }
                 }
                 // set global from lastprivate variables
                 if (amLast) {
                 }
                 // set global from reduction variables
                 if (jomp.runtime.OMP.getThreadNum(__omp_me) == 0) {
                 }
                 } // OMP FOR BLOCK ENDS

                 { // OMP FOR BLOCK BEGINS
                 // copy of firstprivate variables, initialized
                 // copy of lastprivate variables
                 // variables to hold result of reduction
                 boolean amLast=false;
                 {
                   // firstprivate variables + init
                   // [last]private variables
                   // reduction variables + init to default
                   // -------------------------------------
                   jomp.runtime.LoopData __omp_WholeData5 = new jomp.runtime.LoopData();
                   jomp.runtime.LoopData __omp_ChunkData4 = new jomp.runtime.LoopData();
                   __omp_WholeData5.start = (long)( 0);
                   __omp_WholeData5.stop = (long)( n);
                   __omp_WholeData5.step = (long)(1);
                   jomp.runtime.OMP.setChunkStatic(__omp_WholeData5);
                   while(!__omp_ChunkData4.isLast && jomp.runtime.OMP.getLoopStatic(__omp_me, __omp_WholeData5, __omp_ChunkData4)) {
                   for(;;) {
                     if(__omp_WholeData5.step > 0) {
                        if(__omp_ChunkData4.stop > __omp_WholeData5.stop) __omp_ChunkData4.stop = __omp_WholeData5.stop;
                        if(__omp_ChunkData4.start >= __omp_WholeData5.stop) break;
                     } else {
                        if(__omp_ChunkData4.stop < __omp_WholeData5.stop) __omp_ChunkData4.stop = __omp_WholeData5.stop;
                        if(__omp_ChunkData4.start > __omp_WholeData5.stop) break;
                     }
                     for(int i = (int)__omp_ChunkData4.start; i < __omp_ChunkData4.stop; i += __omp_ChunkData4.step) {
                       // OMP USER CODE BEGINS
 {
			out[i] = y[i][n - 1];
		}
                       // OMP USER CODE ENDS
                       if (i == (__omp_WholeData5.stop-1)) amLast = true;
                     } // of for 
                     if(__omp_ChunkData4.startStep == 0)
                       break;
                     __omp_ChunkData4.start += __omp_ChunkData4.startStep;
                     __omp_ChunkData4.stop += __omp_ChunkData4.startStep;
                   } // of for(;;)
                   } // of while
                   // call reducer
                   jomp.runtime.OMP.doBarrier(__omp_me);
                   // copy lastprivate variables out
                   if (amLast) {
                   }
                 }
                 // set global from lastprivate variables
                 if (amLast) {
                 }
                 // set global from reduction variables
                 if (jomp.runtime.OMP.getThreadNum(__omp_me) == 0) {
                 }
                 } // OMP FOR BLOCK ENDS

		}
    // OMP USER CODE ENDS
  // call reducer
  // output to _rd_ copy
  if (jomp.runtime.OMP.getThreadNum(__omp_me) == 0) {
  }
  }
}
// OMP PARALLEL REGION INNER CLASS DEFINITION ENDS

}

