
public class PLgemverJomp {


	public static int threadNumber = 4;
	public static double alpha, beta;
	/* Array initialization. */
	public static void init_array(int n, double[][] A, double[] u1,
			double[] v1, double[] u2, double[] v2, double[] w, double[] x,
			double[] y, double[] z) {
		int i;
		int j;

		alpha = (double) 43532;
		beta = (double) 12313;

		for (i = 0; i < n; i++) {
			u1[i] = i;
			u2[i] = (i + 1) / n / 2.0;
			v1[i] = (i + 1) / n / 4.0;
			v2[i] = (i + 1) / n / 6.0;
			y[i] = (i + 1) / n / 8.0;
			z[i] = (i + 1) / n / 9.0;
			x[i] = 0.0;
			w[i] = 0.0;
			for (j = 0; j < n; j++) {
				A[i][j] = ((double) i * j) / n;
			}
		}
	}

	/*
	 * DCE code. Must scan the entire live-out data. Can be used also to check
	 * the correctness of the output.
	 */
	public static void print_array(int n, double[] w) {
		int i;

		for (i = 0; i < n; i++) {
			System.out.printf("%.2f ", w[i]);
			if (i % 20 == 0) {
				System.out.println();
			}
		}
	}

	/*
	 * Main computational kernel. The whole function will be timed, including
	 * the call and return.
	 */
	public static void kernel_gemver(int n, double alpha, double beta,
			double[][] A, double[] u1, double[] v1, double[] u2, double[] v2,
			double[] w, double[] x, double[] y, double[] z) {

// OMP PARALLEL BLOCK BEGINS
{
  __omp_Class0 __omp_Object0 = new __omp_Class0();
  // shared variables
  __omp_Object0.n = n;
  __omp_Object0.w = w;
  __omp_Object0.x = x;
  __omp_Object0.y = y;
  __omp_Object0.z = z;
  __omp_Object0.alpha = alpha;
  __omp_Object0.beta = beta;
  __omp_Object0.A = A;
  __omp_Object0.u1 = u1;
  __omp_Object0.v1 = v1;
  __omp_Object0.u2 = u2;
  __omp_Object0.v2 = v2;
  // firstprivate variables
  try {
    jomp.runtime.OMP.doParallel(__omp_Object0);
  } catch(Throwable __omp_exception) {
    System.err.println("OMP Warning: Illegal thread exception ignored!");
    System.err.println(__omp_exception);
  }
  // reduction variables
  // shared variables
  n = __omp_Object0.n;
  w = __omp_Object0.w;
  x = __omp_Object0.x;
  y = __omp_Object0.y;
  z = __omp_Object0.z;
  alpha = __omp_Object0.alpha;
  beta = __omp_Object0.beta;
  A = __omp_Object0.A;
  u1 = __omp_Object0.u1;
  v1 = __omp_Object0.v1;
  u2 = __omp_Object0.u2;
  v2 = __omp_Object0.v2;
}
// OMP PARALLEL BLOCK ENDS

		// #pragma endscop
	}

	public static void main(String[] args) {
		/* Retrieve problem size. */
		String dataSet;
		if (args.length == 2) {
			dataSet = args[0].toUpperCase();
			threadNumber = Integer.parseInt(args[1]);
		} else if (args.length == 1) {
			dataSet = args[0].toUpperCase();
		} else {
			dataSet = "STANDARD";
		}
		int N;
		switch (dataSet) {
		case "MINI":
			N = 32;
			break;
		case "SMALL":
			N = 500;
			break;
		case "STANDARD":
		default:
			N = 4000;
			break;
		case "LARGE":
			N = 8000;
			break;
		case "EXTRALARGE":
			N = 100000;
			break;
		}
		int n = N;

		/* Variable declaration/allocation. */
		double[][] A = new double[n][n];
		double[] u1 = new double[n];
		double[] v1 = new double[n];
		double[] u2 = new double[n];
		double[] v2 = new double[n];
		double[] w = new double[n];
		double[] x = new double[n];
		double[] y = new double[n];
		double[] z = new double[n];

		/* Initialize array(s). */
		PLgemverJomp.init_array(n, A, u1, v1, u2, v2,
				w, x, y, z);

		/* Start timer. */
		long pre = System.currentTimeMillis();

		/* Run kernel. */
		PLgemverJomp.kernel_gemver(n, alpha, beta, A, u1, v1, u2, v2, w, x, y, z);

		/* Stop and print timer. */
		long pos = System.currentTimeMillis();
		System.out.print(pos - pre);

		/*
		 * Prevent dead-code elimination. All live-out data must be printed by
		 * the function call in argument.
		 */
		// PLgemverJomp.print_array(n, w);

	}

// OMP PARALLEL REGION INNER CLASS DEFINITION BEGINS
private static class __omp_Class0 extends jomp.runtime.BusyTask {
  // shared variables
  int n;
  double [ ] w;
  double [ ] x;
  double [ ] y;
  double [ ] z;
  double alpha;
  double beta;
  double [ ] [ ] A;
  double [ ] u1;
  double [ ] v1;
  double [ ] u2;
  double [ ] v2;
  // firstprivate variables
  // variables to hold results of reduction

  public void go(int __omp_me) throws Throwable {
  // firstprivate variables + init
  // private variables
  // reduction variables, init to default
    // OMP USER CODE BEGINS
 
		{
                 { // OMP FOR BLOCK BEGINS
                 // copy of firstprivate variables, initialized
                 // copy of lastprivate variables
                 // variables to hold result of reduction
                 boolean amLast=false;
                 {
                   // firstprivate variables + init
                   // [last]private variables
                   // reduction variables + init to default
                   // -------------------------------------
                   jomp.runtime.LoopData __omp_WholeData2 = new jomp.runtime.LoopData();
                   jomp.runtime.LoopData __omp_ChunkData1 = new jomp.runtime.LoopData();
                   __omp_WholeData2.start = (long)( 0);
                   __omp_WholeData2.stop = (long)( n);
                   __omp_WholeData2.step = (long)(1);
                   jomp.runtime.OMP.setChunkStatic(__omp_WholeData2);
                   while(!__omp_ChunkData1.isLast && jomp.runtime.OMP.getLoopStatic(__omp_me, __omp_WholeData2, __omp_ChunkData1)) {
                   for(;;) {
                     if(__omp_WholeData2.step > 0) {
                        if(__omp_ChunkData1.stop > __omp_WholeData2.stop) __omp_ChunkData1.stop = __omp_WholeData2.stop;
                        if(__omp_ChunkData1.start >= __omp_WholeData2.stop) break;
                     } else {
                        if(__omp_ChunkData1.stop < __omp_WholeData2.stop) __omp_ChunkData1.stop = __omp_WholeData2.stop;
                        if(__omp_ChunkData1.start > __omp_WholeData2.stop) break;
                     }
                     for(int i = (int)__omp_ChunkData1.start; i < __omp_ChunkData1.stop; i += __omp_ChunkData1.step) {
                       // OMP USER CODE BEGINS
 {
			for (int j = 0; j < n; j++) {
				A[i][j] = A[i][j] + u1[i] * v1[j] + u2[i] * v2[j];
			}
		}
                       // OMP USER CODE ENDS
                       if (i == (__omp_WholeData2.stop-1)) amLast = true;
                     } // of for 
                     if(__omp_ChunkData1.startStep == 0)
                       break;
                     __omp_ChunkData1.start += __omp_ChunkData1.startStep;
                     __omp_ChunkData1.stop += __omp_ChunkData1.startStep;
                   } // of for(;;)
                   } // of while
                   // call reducer
                   jomp.runtime.OMP.doBarrier(__omp_me);
                   // copy lastprivate variables out
                   if (amLast) {
                   }
                 }
                 // set global from lastprivate variables
                 if (amLast) {
                 }
                 // set global from reduction variables
                 if (jomp.runtime.OMP.getThreadNum(__omp_me) == 0) {
                 }
                 } // OMP FOR BLOCK ENDS

                 { // OMP FOR BLOCK BEGINS
                 // copy of firstprivate variables, initialized
                 // copy of lastprivate variables
                 // variables to hold result of reduction
                 boolean amLast=false;
                 {
                   // firstprivate variables + init
                   // [last]private variables
                   // reduction variables + init to default
                   // -------------------------------------
                   jomp.runtime.LoopData __omp_WholeData5 = new jomp.runtime.LoopData();
                   jomp.runtime.LoopData __omp_ChunkData4 = new jomp.runtime.LoopData();
                   __omp_WholeData5.start = (long)( 0);
                   __omp_WholeData5.stop = (long)( n);
                   __omp_WholeData5.step = (long)(1);
                   jomp.runtime.OMP.setChunkStatic(__omp_WholeData5);
                   while(!__omp_ChunkData4.isLast && jomp.runtime.OMP.getLoopStatic(__omp_me, __omp_WholeData5, __omp_ChunkData4)) {
                   for(;;) {
                     if(__omp_WholeData5.step > 0) {
                        if(__omp_ChunkData4.stop > __omp_WholeData5.stop) __omp_ChunkData4.stop = __omp_WholeData5.stop;
                        if(__omp_ChunkData4.start >= __omp_WholeData5.stop) break;
                     } else {
                        if(__omp_ChunkData4.stop < __omp_WholeData5.stop) __omp_ChunkData4.stop = __omp_WholeData5.stop;
                        if(__omp_ChunkData4.start > __omp_WholeData5.stop) break;
                     }
                     for(int i = (int)__omp_ChunkData4.start; i < __omp_ChunkData4.stop; i += __omp_ChunkData4.step) {
                       // OMP USER CODE BEGINS
 {
			for (int j = 0; j < n; j++) {
				x[i] = x[i] + beta * A[j][i] * y[j];
			}
		}
                       // OMP USER CODE ENDS
                       if (i == (__omp_WholeData5.stop-1)) amLast = true;
                     } // of for 
                     if(__omp_ChunkData4.startStep == 0)
                       break;
                     __omp_ChunkData4.start += __omp_ChunkData4.startStep;
                     __omp_ChunkData4.stop += __omp_ChunkData4.startStep;
                   } // of for(;;)
                   } // of while
                   // call reducer
                   jomp.runtime.OMP.doBarrier(__omp_me);
                   // copy lastprivate variables out
                   if (amLast) {
                   }
                 }
                 // set global from lastprivate variables
                 if (amLast) {
                 }
                 // set global from reduction variables
                 if (jomp.runtime.OMP.getThreadNum(__omp_me) == 0) {
                 }
                 } // OMP FOR BLOCK ENDS

                 { // OMP FOR BLOCK BEGINS
                 // copy of firstprivate variables, initialized
                 // copy of lastprivate variables
                 // variables to hold result of reduction
                 boolean amLast=false;
                 {
                   // firstprivate variables + init
                   // [last]private variables
                   // reduction variables + init to default
                   // -------------------------------------
                   jomp.runtime.LoopData __omp_WholeData8 = new jomp.runtime.LoopData();
                   jomp.runtime.LoopData __omp_ChunkData7 = new jomp.runtime.LoopData();
                   __omp_WholeData8.start = (long)( 0);
                   __omp_WholeData8.stop = (long)( n);
                   __omp_WholeData8.step = (long)(1);
                   jomp.runtime.OMP.setChunkStatic(__omp_WholeData8);
                   while(!__omp_ChunkData7.isLast && jomp.runtime.OMP.getLoopStatic(__omp_me, __omp_WholeData8, __omp_ChunkData7)) {
                   for(;;) {
                     if(__omp_WholeData8.step > 0) {
                        if(__omp_ChunkData7.stop > __omp_WholeData8.stop) __omp_ChunkData7.stop = __omp_WholeData8.stop;
                        if(__omp_ChunkData7.start >= __omp_WholeData8.stop) break;
                     } else {
                        if(__omp_ChunkData7.stop < __omp_WholeData8.stop) __omp_ChunkData7.stop = __omp_WholeData8.stop;
                        if(__omp_ChunkData7.start > __omp_WholeData8.stop) break;
                     }
                     for(int i = (int)__omp_ChunkData7.start; i < __omp_ChunkData7.stop; i += __omp_ChunkData7.step) {
                       // OMP USER CODE BEGINS
 {
			x[i] = x[i] + z[i];
		}
                       // OMP USER CODE ENDS
                       if (i == (__omp_WholeData8.stop-1)) amLast = true;
                     } // of for 
                     if(__omp_ChunkData7.startStep == 0)
                       break;
                     __omp_ChunkData7.start += __omp_ChunkData7.startStep;
                     __omp_ChunkData7.stop += __omp_ChunkData7.startStep;
                   } // of for(;;)
                   } // of while
                   // call reducer
                   jomp.runtime.OMP.doBarrier(__omp_me);
                   // copy lastprivate variables out
                   if (amLast) {
                   }
                 }
                 // set global from lastprivate variables
                 if (amLast) {
                 }
                 // set global from reduction variables
                 if (jomp.runtime.OMP.getThreadNum(__omp_me) == 0) {
                 }
                 } // OMP FOR BLOCK ENDS

                 { // OMP FOR BLOCK BEGINS
                 // copy of firstprivate variables, initialized
                 // copy of lastprivate variables
                 // variables to hold result of reduction
                 boolean amLast=false;
                 {
                   // firstprivate variables + init
                   // [last]private variables
                   // reduction variables + init to default
                   // -------------------------------------
                   jomp.runtime.LoopData __omp_WholeData11 = new jomp.runtime.LoopData();
                   jomp.runtime.LoopData __omp_ChunkData10 = new jomp.runtime.LoopData();
                   __omp_WholeData11.start = (long)( 0);
                   __omp_WholeData11.stop = (long)( n);
                   __omp_WholeData11.step = (long)(1);
                   jomp.runtime.OMP.setChunkStatic(__omp_WholeData11);
                   while(!__omp_ChunkData10.isLast && jomp.runtime.OMP.getLoopStatic(__omp_me, __omp_WholeData11, __omp_ChunkData10)) {
                   for(;;) {
                     if(__omp_WholeData11.step > 0) {
                        if(__omp_ChunkData10.stop > __omp_WholeData11.stop) __omp_ChunkData10.stop = __omp_WholeData11.stop;
                        if(__omp_ChunkData10.start >= __omp_WholeData11.stop) break;
                     } else {
                        if(__omp_ChunkData10.stop < __omp_WholeData11.stop) __omp_ChunkData10.stop = __omp_WholeData11.stop;
                        if(__omp_ChunkData10.start > __omp_WholeData11.stop) break;
                     }
                     for(int i = (int)__omp_ChunkData10.start; i < __omp_ChunkData10.stop; i += __omp_ChunkData10.step) {
                       // OMP USER CODE BEGINS
 {
			for (int j = 0; j < n; j++) {
				w[i] = w[i] + alpha * A[i][j] * x[j];
			}
		}
                       // OMP USER CODE ENDS
                       if (i == (__omp_WholeData11.stop-1)) amLast = true;
                     } // of for 
                     if(__omp_ChunkData10.startStep == 0)
                       break;
                     __omp_ChunkData10.start += __omp_ChunkData10.startStep;
                     __omp_ChunkData10.stop += __omp_ChunkData10.startStep;
                   } // of for(;;)
                   } // of while
                   // call reducer
                   jomp.runtime.OMP.doBarrier(__omp_me);
                   // copy lastprivate variables out
                   if (amLast) {
                   }
                 }
                 // set global from lastprivate variables
                 if (amLast) {
                 }
                 // set global from reduction variables
                 if (jomp.runtime.OMP.getThreadNum(__omp_me) == 0) {
                 }
                 } // OMP FOR BLOCK ENDS

		}
    // OMP USER CODE ENDS
  // call reducer
  // output to _rd_ copy
  if (jomp.runtime.OMP.getThreadNum(__omp_me) == 0) {
  }
  }
}
// OMP PARALLEL REGION INNER CLASS DEFINITION ENDS

}

