
public class PSfdtd_2dJomp {


	public static int threadNumber = 4;
	/* Array initialization. */
	public static void init_array(int tmax, int nx, int ny, double[][] ex,
			double[][] ey, double[][] hz, double[] _fict_) {
		int i;
		int j;

		for (i = 0; i < tmax; i++) {
			_fict_[i] = (double) i;
		}
		for (i = 0; i < nx; i++) {
			for (j = 0; j < ny; j++) {
				ex[i][j] = ((double) i * (j + 1)) / nx;
				ey[i][j] = ((double) i * (j + 2)) / ny;
				hz[i][j] = ((double) i * (j + 3)) / nx;
			}
		}
	}

	/*
	 * DCE code. Must scan the entire live-out data. Can be used also to check
	 * the correctness of the output.
	 */
	public static void print_array(int nx, int ny, double[][] ex,
			double[][] ey, double[][] hz) {
		int i;
		int j;

		for (i = 0; i < nx; i++) {
			for (j = 0; j < ny; j++) {
				System.out.printf("%.2f ", ex[i][j]);
				System.out.printf("%.2f ", ey[i][j]);
				System.out.printf("%.2f ", hz[i][j]);
				if ((i * nx + j) % 20 == 0) {
					System.out.println();
				}
			}
		}
		System.out.println();
	}

	/*
	 * Main computational kernel. The whole function will be timed, including
	 * the call and return.
	 */
	public static void kernel_fdtd_2d(int tmax, int nx, int ny, double[][] ex,
			double[][] ey, double[][] hz, double[] _fict_) {

// OMP PARALLEL BLOCK BEGINS
{
  __omp_Class0 __omp_Object0 = new __omp_Class0();
  // shared variables
  __omp_Object0.tmax = tmax;
  __omp_Object0.nx = nx;
  __omp_Object0.ny = ny;
  __omp_Object0.ex = ex;
  __omp_Object0.ey = ey;
  __omp_Object0._fict_ = _fict_;
  __omp_Object0.hz = hz;
  // firstprivate variables
  try {
    jomp.runtime.OMP.doParallel(__omp_Object0);
  } catch(Throwable __omp_exception) {
    System.err.println("OMP Warning: Illegal thread exception ignored!");
    System.err.println(__omp_exception);
  }
  // reduction variables
  // shared variables
  tmax = __omp_Object0.tmax;
  nx = __omp_Object0.nx;
  ny = __omp_Object0.ny;
  ex = __omp_Object0.ex;
  ey = __omp_Object0.ey;
  _fict_ = __omp_Object0._fict_;
  hz = __omp_Object0.hz;
}
// OMP PARALLEL BLOCK ENDS

		// #pragma endscop
	}

	public static void main(String[] args) {
		/* Retrieve problem size. */
		String dataSet;
		if (args.length == 2) {
			dataSet = args[0].toUpperCase();
			threadNumber = Integer.parseInt(args[1]);
		} else if (args.length == 1) {
			dataSet = args[0].toUpperCase();
		} else {
			dataSet = "STANDARD";
		}
		int TMAX, NX, NY;
		switch (dataSet) {
		case "MINI":
			TMAX = 2;
			NX = 32;
			NY = 32;
			break;
		case "SMALL":
			TMAX = 10;
			NX = 500;
			NY = 500;
			break;
		case "STANDARD":
		default:
			TMAX = 50;
			NX = 1000;
			NY = 1000;
			break;
		case "LARGE":
			TMAX = 50;
			NX = 2000;
			NY = 2000;
			break;
		case "EXTRALARGE":
			TMAX = 100;
			NX = 4000;
			NY = 4000;
			break;
		}
		int tmax = TMAX;
		int nx = NX;
		int ny = NY;

		/* Variable declaration/allocation. */
		double[][] ex = new double[nx][ny];
		double[][] ey = new double[nx][ny];
		double[][] hz = new double[nx][ny];
		double[] _fict_ = new double[tmax];

		/* Initialize array(s). */
		PSfdtd_2dJomp.init_array(tmax, nx, ny, ex, ey, hz, _fict_);

		/* Start timer. */
		long pre = System.currentTimeMillis();

		/* Run kernel. */
		PSfdtd_2dJomp.kernel_fdtd_2d(tmax, nx, ny, ex, ey, hz, _fict_);

		/* Stop and print timer. */
		long pos = System.currentTimeMillis();
		System.out.print(pos - pre);

		/*
		 * Prevent dead-code elimination. All live-out data must be printed by
		 * the function call in argument.
		 */
		// PSfdtd_2dJomp.print_array(nx, ny, ex, ey, hz);

	}

// OMP PARALLEL REGION INNER CLASS DEFINITION BEGINS
private static class __omp_Class0 extends jomp.runtime.BusyTask {
  // shared variables
  int tmax;
  int nx;
  int ny;
  double [ ] [ ] ex;
  double [ ] [ ] ey;
  double [ ] _fict_;
  double [ ] [ ] hz;
  // firstprivate variables
  // variables to hold results of reduction

  public void go(int __omp_me) throws Throwable {
  // firstprivate variables + init
  // private variables
  // reduction variables, init to default
    // OMP USER CODE BEGINS

		{
                 { // OMP FOR BLOCK BEGINS
                 // copy of firstprivate variables, initialized
                 // copy of lastprivate variables
                 // variables to hold result of reduction
                 boolean amLast=false;
                 {
                   // firstprivate variables + init
                   // [last]private variables
                   // reduction variables + init to default
                   // -------------------------------------
                   jomp.runtime.LoopData __omp_WholeData2 = new jomp.runtime.LoopData();
                   jomp.runtime.LoopData __omp_ChunkData1 = new jomp.runtime.LoopData();
                   __omp_WholeData2.start = (long)( 0);
                   __omp_WholeData2.stop = (long)( tmax);
                   __omp_WholeData2.step = (long)(1);
                   jomp.runtime.OMP.setChunkStatic(__omp_WholeData2);
                   while(!__omp_ChunkData1.isLast && jomp.runtime.OMP.getLoopStatic(__omp_me, __omp_WholeData2, __omp_ChunkData1)) {
                   for(;;) {
                     if(__omp_WholeData2.step > 0) {
                        if(__omp_ChunkData1.stop > __omp_WholeData2.stop) __omp_ChunkData1.stop = __omp_WholeData2.stop;
                        if(__omp_ChunkData1.start >= __omp_WholeData2.stop) break;
                     } else {
                        if(__omp_ChunkData1.stop < __omp_WholeData2.stop) __omp_ChunkData1.stop = __omp_WholeData2.stop;
                        if(__omp_ChunkData1.start > __omp_WholeData2.stop) break;
                     }
                     for(int t = (int)__omp_ChunkData1.start; t < __omp_ChunkData1.stop; t += __omp_ChunkData1.step) {
                       // OMP USER CODE BEGINS
 {
			for (int j = 0; j < ny; j++) {
				ey[0][j] = _fict_[t];
			}
			for (int i = 1; i < nx; i++) {
				for (int j = 0; j < ny; j++) {
					ey[i][j] = ey[i][j] - 0.5 * (hz[i][j] - hz[i - 1][j]);
				}
			}
			for (int i = 0; i < nx; i++) {
				for (int j = 1; j < ny; j++) {
					ex[i][j] = ex[i][j] - 0.5 * (hz[i][j] - hz[i][j - 1]);
				}
			}
			for (int i = 0; i < nx - 1; i++) {
				for (int j = 0; j < ny - 1; j++) {
					hz[i][j] = hz[i][j]
							- 0.7
							* (ex[i][j + 1] - ex[i][j] + ey[i + 1][j] - ey[i][j]);
				}
			}
		}
                       // OMP USER CODE ENDS
                       if (t == (__omp_WholeData2.stop-1)) amLast = true;
                     } // of for 
                     if(__omp_ChunkData1.startStep == 0)
                       break;
                     __omp_ChunkData1.start += __omp_ChunkData1.startStep;
                     __omp_ChunkData1.stop += __omp_ChunkData1.startStep;
                   } // of for(;;)
                   } // of while
                   // call reducer
                   jomp.runtime.OMP.doBarrier(__omp_me);
                   // copy lastprivate variables out
                   if (amLast) {
                   }
                 }
                 // set global from lastprivate variables
                 if (amLast) {
                 }
                 // set global from reduction variables
                 if (jomp.runtime.OMP.getThreadNum(__omp_me) == 0) {
                 }
                 } // OMP FOR BLOCK ENDS

		}
    // OMP USER CODE ENDS
  // call reducer
  // output to _rd_ copy
  if (jomp.runtime.OMP.getThreadNum(__omp_me) == 0) {
  }
  }
}
// OMP PARALLEL REGION INNER CLASS DEFINITION ENDS

}

